/*
 * Copyright (c) 2001-2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: LaunchAction.java 3185 2006-09-01 05:30:41Z jmettraux $
 */

//
// LaunchAction.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.wlactions;

import javax.security.auth.Subject;

import openwfe.org.ApplicationContext;
import openwfe.org.auth.actions.OwfeAction;
import openwfe.org.engine.workitem.LaunchItem;
import openwfe.org.engine.dispatch.DispatchingException;
import openwfe.org.engine.participants.Participant;
import openwfe.org.engine.participants.ParticipantMap;
import openwfe.org.worklist.auth.LaunchPermission;


/**
 * The action of launching a flow
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-09-01 07:30:41 +0200 (Fri, 01 Sep 2006) $
 * <br>$Id: LaunchAction.java 3185 2006-09-01 05:30:41Z jmettraux $ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class LaunchAction

    //implements java.security.PrivilegedExceptionAction
    extends OwfeAction

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(LaunchAction.class.getName());

    //
    // FIELDS

    /*
    private ApplicationContext worklistContext = null;
    private Subject subject = null;
    private String engineId = null;
    private LaunchItem item = null;
    */

    //
    // CONSTRUCTORS

    /*
    public LaunchAction 
        (final ApplicationContext worklistContext, 
         final Subject subject, 
         final String engineId,
         final LaunchItem item)
    {
        this.worklistContext = worklistContext;
        this.subject = subject;

        this.engineId = engineId;
        this.item = item;

        log.debug("() engineId is >"+this.engineId+"<");
    }
    */

    //
    // METHODS

    protected String getEngineId ()
    {
        return (String)this.getArgs()[0];
    }

    protected LaunchItem getLaunchItem ()
    {
        return (LaunchItem)this.getArgs()[1];
    }

    public Object run () 
        throws DispatchingException
    {

        if (this.getLaunchItem().getWorkflowDefinitionUrl() == null)
        {
            throw new DispatchingException
                ("LaunchItem with workflowDefinitionUrl field not set. "+
                 "Cannot launch.");
        }

        //
        // security control is performed here...

        java.security.AccessController.checkPermission
            (LaunchPermission.newLaunchPermission
                (determinePermissionName()));

        //
        // find participant and dispatch
        
        final ParticipantMap pMap = openwfe.org.engine.Definitions
            .getParticipantMap(this.getContext());

        final Participant p = pMap.get(this.getEngineId());

        if (p == null)
        {
            throw new DispatchingException 
                ("Cannot launch flow on unknown engine '"+
                 this.getEngineId()+"'");
        }

        //log.debug("run() dispatching launchitem to engine...");

        return 
            p.dispatch(this.getContext(), this.getLaunchItem());
    }

    protected String determinePermissionName ()
    {
        StringBuffer sb = new StringBuffer();

        sb.append(this.getEngineId());
        sb.append("::");
        sb.append(this.getLaunchItem().getWorkflowDefinitionUrl());

        return sb.toString();
    }

}
